@@ -25,7 +25,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <stdio.h>
@@ -41,8 +41,6 @@
 
 #ifdef HAVE_MMAP
 #include <sys/mman.h>
-#else
-#warning Building xdgmime without MMAP support. Binary "mime.info" cache files will not be used.
 #endif
 
 #include <sys/stat.h>
@@ -72,13 +70,11 @@
 #endif
 
 #define MAJOR_VERSION 1
-#define MINOR_VERSION_MIN 1
-#define MINOR_VERSION_MAX 2
+#define MINOR_VERSION 0
 
 struct _XdgMimeCache
 {
   int ref_count;
-  int minor;
 
   size_t  size;
   char   *buffer;
@@ -117,7 +113,6 @@ _xdg_mime_cache_new_from_file (const char *file_name)
   int fd = -1;
   struct stat st;
   char *buffer = NULL;
-  int minor;
 
   /* Open the file and map it into memory */
   fd = open (file_name, O_RDONLY|_O_BINARY, 0);
@@ -133,11 +128,9 @@ _xdg_mime_cache_new_from_file (const char *file_name)
   if (buffer == MAP_FAILED)
     goto done;
 
-  minor = GET_UINT16 (buffer, 2);
   /* Verify version */
   if (GET_UINT16 (buffer, 0) != MAJOR_VERSION ||
-      (minor < MINOR_VERSION_MIN ||
-       minor > MINOR_VERSION_MAX))
+      GET_UINT16 (buffer, 2) != MINOR_VERSION)
     {
       munmap (buffer, st.st_size);
 
@@ -145,7 +138,6 @@ _xdg_mime_cache_new_from_file (const char *file_name)
     }
   
   cache = (XdgMimeCache *) malloc (sizeof (XdgMimeCache));
-  cache->minor = minor;
   cache->ref_count = 1;
   cache->buffer = buffer;
   cache->size = st.st_size;
@@ -173,7 +165,7 @@ cache_magic_matchlet_compare_to_data (XdgMimeCache *cache,
   
   int i, j;
 
-  for (i = range_start; i < range_start + range_length; i++)
+  for (i = range_start; i <= range_start + range_length; i++)
     {
       int valid_matchlet = TRUE;
       
@@ -305,7 +297,7 @@ cache_magic_lookup_data (XdgMimeCache *cache,
 	  for (n = 0; n < n_mime_types; n++)
 	    {
 	      if (mime_types[n] && 
-		  _xdg_mime_mime_type_equal (mime_types[n], non_match))
+		  xdg_mime_mime_type_equal (mime_types[n], non_match))
 		mime_types[n] = NULL;
 	    }
 	}
@@ -352,16 +344,10 @@ cache_alias_lookup (const char *alias)
   return NULL;
 }
 
-typedef struct {
-  const char *mime;
-  int weight;
-} MimeWeight;
-
 static int
 cache_glob_lookup_literal (const char *file_name,
 			   const char *mime_types[],
-			   int         n_mime_types,
-			   int         case_sensitive_check)
+			   int         n_mime_types)
 {
   const char *ptr;
   int i, min, max, mid, cmp;
@@ -379,28 +365,20 @@ cache_glob_lookup_literal (const char *file_name,
 	{
 	  mid = (min + max) / 2;
 
-	  offset = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * mid);
+	  offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * mid);
 	  ptr = cache->buffer + offset;
 	  cmp = strcmp (ptr, file_name);
-
+	  
 	  if (cmp < 0)
 	    min = mid + 1;
 	  else if (cmp > 0)
 	    max = mid - 1;
 	  else
 	    {
-	      int weight = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * mid + 8);
-	      int case_sensitive = weight & 0x100;
-	      weight = weight & 0xff;
-
-	      if (case_sensitive_check || !case_sensitive)
-		{
-		  offset = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * mid + 4);
-		  mime_types[0] = (const char *)(cache->buffer + offset);
-
-		  return 1;
-		}
-	      return 0;
+	      offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * mid + 4);
+	      mime_types[0] = (const char *)(cache->buffer + offset);
+	      
+	      return 1;
 	    }
 	}
     }
@@ -410,7 +388,7 @@ cache_glob_lookup_literal (const char *file_name,
 
 static int
 cache_glob_lookup_fnmatch (const char *file_name,
-			   MimeWeight  mime_types[],
+			   const char *mime_types[],
 			   int         n_mime_types)
 {
   const char *mime_type;
@@ -428,118 +406,107 @@ cache_glob_lookup_fnmatch (const char *file_name,
 
       for (j = 0; j < n_entries && n < n_mime_types; j++)
 	{
-	  xdg_uint32_t offset = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * j);
-	  xdg_uint32_t mimetype_offset = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * j + 4);
-	  int weight = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * j + 8);
-	  weight = weight & 0xff;
+	  xdg_uint32_t offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * j);
+	  xdg_uint32_t mimetype_offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * j + 4);
 	  ptr = cache->buffer + offset;
 	  mime_type = cache->buffer + mimetype_offset;
 
 	  /* FIXME: Not UTF-8 safe */
 	  if (fnmatch (ptr, file_name, 0) == 0)
-	    {
-	      mime_types[n].mime = mime_type;
-	      mime_types[n].weight = weight;
-	      n++;
-	    }
+	    mime_types[n++] = mime_type;
 	}
 
-      if (n == n_mime_types)
-	break;
+      if (n > 0)
+	return n;
     }
-
-  return n;
+  
+  return 0;
 }
 
 static int
-cache_glob_node_lookup_suffix (XdgMimeCache  *cache,
-			       xdg_uint32_t   n_entries,
-			       xdg_uint32_t   offset,
-			       const char    *file_name,
-			       int            len,
-			       int            case_sensitive_check,
-			       MimeWeight     mime_types[],
-			       int            n_mime_types)
+cache_glob_node_lookup_suffix (XdgMimeCache *cache,
+			       xdg_uint32_t  n_entries,
+			       xdg_uint32_t  offset,
+			       const char   *suffix, 
+			       int           ignore_case,
+			       const char   *mime_types[],
+			       int           n_mime_types)
 {
   xdg_unichar_t character;
   xdg_unichar_t match_char;
   xdg_uint32_t mimetype_offset;
   xdg_uint32_t n_children;
   xdg_uint32_t child_offset; 
-  int weight;
-  int case_sensitive;
 
   int min, max, mid, n, i;
 
-  character = file_name[len - 1];
-
-  assert (character != 0);
+  character = _xdg_utf8_to_ucs4 (suffix);
+  if (ignore_case)
+    character = _xdg_ucs4_to_lower (character);
 
   min = 0;
   max = n_entries - 1;
   while (max >= min)
     {
       mid = (min + max) /  2;
-      match_char = GET_UINT32 (cache->buffer, offset + 12 * mid);
+
+      match_char = GET_UINT32 (cache->buffer, offset + 16 * mid);
+
       if (match_char < character)
 	min = mid + 1;
       else if (match_char > character)
 	max = mid - 1;
       else 
 	{
-          len--;
-          n = 0;
-          n_children = GET_UINT32 (cache->buffer, offset + 12 * mid + 4);
-          child_offset = GET_UINT32 (cache->buffer, offset + 12 * mid + 8);
-      
-          if (len > 0)
-            {
-              n = cache_glob_node_lookup_suffix (cache, 
-                                                 n_children, child_offset,
-                                                 file_name, len, 
-                                                 case_sensitive_check,
-                                                 mime_types,
-                                                 n_mime_types);
-            }
-          if (n == 0)
-            {
+	  suffix = _xdg_utf8_next_char (suffix);
+	  if (*suffix == '\0')
+	    {
+	      mimetype_offset = GET_UINT32 (cache->buffer, offset + 16 * mid + 4);
+	      n = 0;
+	      if (mimetype_offset)
+		mime_types[n++] = cache->buffer + mimetype_offset;
+	      
+	      n_children = GET_UINT32 (cache->buffer, offset + 16 * mid + 8);
+	      child_offset = GET_UINT32 (cache->buffer, offset + 16 * mid + 12);
 	      i = 0;
 	      while (n < n_mime_types && i < n_children)
 		{
-		  match_char = GET_UINT32 (cache->buffer, child_offset + 12 * i);
+		  match_char = GET_UINT32 (cache->buffer, child_offset + 16 * i);
+		  mimetype_offset = GET_UINT32 (cache->buffer, offset + 16 * i + 4);
 		  if (match_char != 0)
 		    break;
 
-		  mimetype_offset = GET_UINT32 (cache->buffer, child_offset + 12 * i + 4);
-		  weight = GET_UINT32 (cache->buffer, child_offset + 12 * i + 8);
-		  case_sensitive = weight & 0x100;
-		  weight = weight & 0xff;
-
-		  if (case_sensitive_check || !case_sensitive)
-		    {
-		      mime_types[n].mime = cache->buffer + mimetype_offset;
-		      mime_types[n].weight = weight;
-		      n++;
-		    }
+		  mime_types[n++] = cache->buffer + mimetype_offset;
 		  i++;
 		}
+
+	      return n;
+	    }
+	  else
+	    {
+	      n_children = GET_UINT32 (cache->buffer, offset + 16 * mid + 8);
+	      child_offset = GET_UINT32 (cache->buffer, offset + 16 * mid + 12);
+      
+	      return cache_glob_node_lookup_suffix (cache, 
+						    n_children, child_offset,
+						    suffix, ignore_case,
+						    mime_types,
+						    n_mime_types);
 	    }
-	  return n;
 	}
     }
+
   return 0;
 }
 
 static int
-cache_glob_lookup_suffix (const char *file_name,
-			  int         len,
+cache_glob_lookup_suffix (const char *suffix, 
 			  int         ignore_case,
-			  MimeWeight  mime_types[],
+			  const char *mime_types[],
 			  int         n_mime_types)
 {
   int i, n;
 
-  n = 0;
   for (i = 0; _caches[i]; i++)
     {
       XdgMimeCache *cache = _caches[i];
@@ -548,126 +515,90 @@ cache_glob_lookup_suffix (const char *file_name,
       xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
       xdg_uint32_t offset = GET_UINT32 (cache->buffer, list_offset + 4);
 
-      n += cache_glob_node_lookup_suffix (cache, 
-	 				  n_entries, offset, 
-					  file_name, len,
-					  ignore_case,
-					  mime_types + n,
-					  n_mime_types - n);
-      if (n == n_mime_types)
-	break;
+      n = cache_glob_node_lookup_suffix (cache, 
+					 n_entries, offset, 
+					 suffix, ignore_case,
+					 mime_types,
+					 n_mime_types);
+      if (n > 0)
+	return n;
     }
 
-  return n;
+  return 0;
 }
 
-static int compare_mime_weight (const void *a, const void *b)
+static void
+find_stopchars (char *stopchars)
 {
-  const MimeWeight *aa = (const MimeWeight *)a;
-  const MimeWeight *bb = (const MimeWeight *)b;
-
-  return bb->weight - aa->weight;
-}
-
-#define ISUPPER(c)		((c) >= 'A' && (c) <= 'Z')
-static char *
-ascii_tolower (const char *str)
-{
-  char *p, *lower;
-
-  lower = strdup (str);
-  p = lower;
-  while (*p != 0)
+  int i, j, k, l;
+ 
+  k = 0;
+  for (i = 0; _caches[i]; i++)
     {
-      char c = *p;
-      *p++ = ISUPPER (c) ? c - 'A' + 'a' : c;
+      XdgMimeCache *cache = _caches[i];
+
+      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, 16);
+      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
+      xdg_uint32_t offset = GET_UINT32 (cache->buffer, list_offset + 4);
+
+      for (j = 0; j < n_entries; j++)
+	{
+	  xdg_uint32_t match_char = GET_UINT32 (cache->buffer, offset);
+	  
+	  if (match_char < 128)
+	    {
+	      for (l = 0; l < k; l++)
+		if (stopchars[l] == match_char)
+		  break;
+	      if (l == k)
+		{
+		  stopchars[k] = (char) match_char;
+		  k++;
+		}
+	    }
+
+	  offset += 16;
+	}
     }
-  return lower;
+
+  stopchars[k] = '\0';
 }
 
 static int
-filter_out_dupes (MimeWeight mimes[], int n_mimes)
-{
-  int last;
-  int i, j;
-
-  last = n_mimes;
-
-  for (i = 0; i < last; i++)
-    {
-      j = i + 1;
-      while (j < last)
-        {
-          if (strcmp (mimes[i].mime, mimes[j].mime) == 0)
-            {
-              mimes[i].weight = MAX (mimes[i].weight, mimes[j].weight);
-              last--;
-              mimes[j].mime = mimes[last].mime;
-              mimes[j].weight = mimes[last].weight;
-            }
-          else
-            j++;
-        }
-    }
-
-  return last;
-}
-
-static int
-cache_glob_lookup_file_name (const char *file_name,
+cache_glob_lookup_file_name (const char *file_name, 
 			     const char *mime_types[],
 			     int         n_mime_types)
 {
+  const char *ptr;
+  char stopchars[128];
   int n;
-  MimeWeight mimes[10];
-  int n_mimes = 10;
-  int i;
-  int len;
-  char *lower_case;
-  int try_lower_case;
-
-  assert (file_name != NULL && n_mime_types > 0);
+  
+  assert (file_name != NULL);
 
   /* First, check the literals */
-
-  lower_case = ascii_tolower (file_name);
-
-  n = cache_glob_lookup_literal (lower_case, mime_types, n_mime_types, FALSE);
+  n = cache_glob_lookup_literal (file_name, mime_types, n_mime_types);
   if (n > 0)
+    return n;
+
+  find_stopchars (stopchars);
+
+  /* Next, check suffixes */
+  ptr = strpbrk (file_name, stopchars);
+  while (ptr)
     {
-      free (lower_case);
-      return n;
+      n = cache_glob_lookup_suffix (ptr, FALSE, mime_types, n_mime_types);
+      if (n > 0)
+	return n;
+      
+      n = cache_glob_lookup_suffix (ptr, TRUE, mime_types, n_mime_types);
+      if (n > 0)
+	return n;
+
+      ptr = strpbrk (ptr + 1, stopchars);
     }
-
-  n = cache_glob_lookup_literal (file_name, mime_types, n_mime_types, TRUE);
-  if (n > 0)
-    {
-      free (lower_case);
-      return n;
-    }
-
-  len = strlen (file_name);
-  n = cache_glob_lookup_suffix (lower_case, len, FALSE, mimes, n_mimes);
-  if (n < 2)
-    n += cache_glob_lookup_suffix (file_name, len, TRUE, mimes + n, n_mimes - n);
-
-  free (lower_case);
-
+  
   /* Last, try fnmatch */
-  if (n < 2)
-    n += cache_glob_lookup_fnmatch (file_name, mimes + n, n_mimes - n);
-
-  n = filter_out_dupes (mimes, n);
-
-  qsort (mimes, n, sizeof (MimeWeight), compare_mime_weight);
-
-  if (n_mime_types < n)
-    n = n_mime_types;
-
-  for (i = 0; i < n; i++)
-    mime_types[i] = mimes[i].mime;
-
-  return n;
+  return cache_glob_lookup_fnmatch (file_name, mime_types, n_mime_types);
 }
 
 int
@@ -746,7 +677,7 @@ _xdg_mime_cache_get_mime_type_for_file (const char  *file_name,
 					struct stat *statbuf)
 {
   const char *mime_type;
-  const char *mime_types[10];
+  const char *mime_types[2];
   FILE *file;
   unsigned char *data;
   int max_extent;
@@ -762,7 +693,7 @@ _xdg_mime_cache_get_mime_type_for_file (const char  *file_name,
     return NULL;
 
   base_name = _xdg_get_base_name (file_name);
-  n = cache_glob_lookup_file_name (base_name, mime_types, 10);
+  n = cache_glob_lookup_file_name (base_name, mime_types, 2);
 
   if (n == 1)
     return mime_types[0];
@@ -934,7 +865,7 @@ _xdg_mime_cache_unalias_mime_type (const char *mime)
 char **
 _xdg_mime_cache_list_mime_parents (const char *mime)
 {
-  int i, j, k, l, p;
+  int i, j, k, p;
   char *all_parents[128]; /* we'll stop at 128 */ 
   char **result;
 
@@ -961,25 +892,14 @@ _xdg_mime_cache_list_mime_parents (const char *mime)
 	      for (k = 0; k < n_parents && p < 127; k++)
 		{
 		  parent_mime_offset = GET_UINT32 (cache->buffer, parents_offset + 4 + 4 * k);
-
-		  /* Don't add same parent multiple times.
-		   * This can happen for instance if the same type is listed in multiple directories
-		   */
-		  for (l = 0; l < p; l++)
-		    {
-		      if (strcmp (all_parents[l], cache->buffer + parent_mime_offset) == 0)
-			break;
-		    }
-
-		  if (l == p)
-		    all_parents[p++] = cache->buffer + parent_mime_offset;
+		  all_parents[p++] = cache->buffer + parent_mime_offset;
 		}
 
 	      break;
 	    }
 	}
     }
-  all_parents[p++] = NULL;
+  all_parents[p++] = 0;
   
   result = (char **) malloc (p * sizeof (char *));
   memcpy (result, all_parents, p * sizeof (char *));
@@ -987,100 +907,3 @@ _xdg_mime_cache_list_mime_parents (const char *mime)
   return result;
 }
 
-static const char *
-cache_lookup_icon (const char *mime, int header)
-{
-  const char *ptr;
-  int i, min, max, mid, cmp;
-
-  for (i = 0; _caches[i]; i++)
-    {
-      XdgMimeCache *cache = _caches[i];
-      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, header);
-      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
-      xdg_uint32_t offset;
-
-      min = 0; 
-      max = n_entries - 1;
-      while (max >= min) 
-        {
-          mid = (min + max) / 2;
-
-          offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * mid);
-          ptr = cache->buffer + offset;
-          cmp = strcmp (ptr, mime);
-         
-          if (cmp < 0)
-            min = mid + 1;
-          else if (cmp > 0)
-            max = mid - 1;
-          else
-            {
-              offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * mid + 4);
-              return cache->buffer + offset;
-            }
-        }
-    }
-
-  return NULL;
-}
-
-const char *
-_xdg_mime_cache_get_generic_icon (const char *mime)
-{
-  return cache_lookup_icon (mime, 36);
-}
-
-const char *
-_xdg_mime_cache_get_icon (const char *mime)
-{
-  return cache_lookup_icon (mime, 32);
-}
-
-static void
-dump_glob_node (XdgMimeCache *cache,
-		xdg_uint32_t  offset,
-		int           depth)
-{
-  xdg_unichar_t character;
-  xdg_uint32_t mime_offset;
-  xdg_uint32_t n_children;
-  xdg_uint32_t child_offset;
-  int i;
-
-  character = GET_UINT32 (cache->buffer, offset);
-  mime_offset = GET_UINT32 (cache->buffer, offset + 4);
-  n_children = GET_UINT32 (cache->buffer, offset + 8);
-  child_offset = GET_UINT32 (cache->buffer, offset + 12);
-  for (i = 0; i < depth; i++)
-    printf (" ");
-  printf ("%c", character);
-  if (mime_offset)
-    printf (" - %s", cache->buffer + mime_offset);
-  printf ("\n");
-  if (child_offset)
-  {
-    for (i = 0; i < n_children; i++)
-      dump_glob_node (cache, child_offset + 20 * i, depth + 1);
-  }
-}
-
-void
-_xdg_mime_cache_glob_dump (void)
-{
-  int i, j;
-  for (i = 0; _caches[i]; i++)
-  {
-    XdgMimeCache *cache = _caches[i];
-    xdg_uint32_t list_offset;
-    xdg_uint32_t n_entries;
-    xdg_uint32_t offset;
-    list_offset = GET_UINT32 (cache->buffer, 16);
-    n_entries = GET_UINT32 (cache->buffer, list_offset);
-    offset = GET_UINT32 (cache->buffer, list_offset + 4);
-    for (j = 0; j < n_entries; j++)
-	    dump_glob_node (cache, offset + 20 * j, 0);
-  }
-}
-
-
