@@ -156,7 +156,7 @@ test_log_msg (GTestLogMsg *msg)
       log_indent += 2;
       break;
     case G_TEST_LOG_SKIP_CASE:
-      if (FALSE && gtester_verbose) /* enable to debug test case skipping logic */
+      if (FALSE && gtester_verbose) // enable to debug test case skipping logic
         {
           gchar *sc = g_strconcat (msg->strings[0], ":", NULL);
           gchar *sleft = g_strdup_printf ("%-68s", sc);
@@ -224,7 +224,7 @@ child_report_cb (GIOChannel  *source,
       (void) status;
     }
   while (length > 0);
-  /* g_print ("LASTIOSTATE: first_read_eof=%d condition=%d\n", first_read_eof, condition); */
+  // g_print ("LASTIOSTATE: first_read_eof=%d condition=%d\n", first_read_eof, condition);
   if (first_read_eof || (condition & (G_IO_ERR | G_IO_HUP)))
     {
       /* if there's no data to read and select() reports an error or hangup,
@@ -273,8 +273,7 @@ launch_test_binary (const char *binary,
   GTestLogBuffer *tlb;
   GSList *slist, *free_list = NULL;
   GError *error = NULL;
-  int argc = 0;
-  const gchar **argv;
+  const gchar *argv[99 + g_slist_length (subtest_args) + g_slist_length (subtest_paths)];
   GPid pid = 0;
   gint report_pipe[2] = { -1, -1 };
   guint child_report_cb_id = 0;
@@ -290,38 +289,11 @@ launch_test_binary (const char *binary,
       return FALSE;
     }
 
-  /* setup argc */
-  for (slist = subtest_args; slist; slist = slist->next)
-    argc++;
-  /* argc++; */
-  if (subtest_quiet)
-    argc++;
-  if (subtest_verbose)
-    argc++;
-  if (!subtest_mode_fatal)
-    argc++;
-  if (subtest_mode_quick)
-    argc++;
-  else
-    argc++;
-  if (subtest_mode_perf)
-    argc++;
-  if (gtester_list_tests)
-    argc++;
-  if (subtest_seedstr)
-    argc++;
-  argc++;
-  if (skip_tests)
-    argc++;
-  for (slist = subtest_paths; slist; slist = slist->next)
-    argc++;
-
   /* setup argv */
-  argv = g_malloc ((argc + 2) * sizeof(gchar *));
   argv[i++] = binary;
   for (slist = subtest_args; slist; slist = slist->next)
     argv[i++] = (gchar*) slist->data;
-  /* argv[i++] = "--debug-log"; */
+  // argv[i++] = "--debug-log";
   if (subtest_quiet)
     argv[i++] = "--quiet";
   if (subtest_verbose)
@@ -371,10 +343,8 @@ launch_test_binary (const char *binary,
       else
         g_warning ("Failed to execute test binary: %s: %s", argv[0], error->message);
       g_clear_error (&error);
-      g_free (argv);
       return FALSE;
     }
-  g_free (argv);
 
   subtest_running = TRUE;
   subtest_io_pending = TRUE;
@@ -395,7 +365,7 @@ launch_test_binary (const char *binary,
          subtest_io_pending ||  /* FALSE once ioc_report closes */
          loop_pending)          /* TRUE while idler, etc are running */
     {
-      /* g_print ("LOOPSTATE: subtest_running=%d subtest_io_pending=%d\n", subtest_running, subtest_io_pending); */
+      // g_print ("LOOPSTATE: subtest_running=%d subtest_io_pending=%d\n", subtest_running, subtest_io_pending);
       /* check for unexpected hangs that are not signalled on report_pipe */
       if (!subtest_running &&   /* child exited */
           subtest_io_pending && /* no EOF detected on report_pipe */
@@ -498,7 +468,7 @@ parse_args (gint    *argc_p,
         {
           gtester_selftest = TRUE;
           argv[i] = NULL;
-          break;        /* stop parsing regular gtester arguments */
+          break;        // stop parsing regular gtester arguments
         }
       else if (strcmp (argv[i], "-h") == 0 || strcmp (argv[i], "--help") == 0)
         {
@@ -672,7 +642,7 @@ main (int    argc,
 
   close (log_fd);
 
-  return testcase_fail_count == 0 ? 0 : 1;
+  return 0;
 }
 
 static void
